SpellShader = {}

BUFFER_TIME = 1500 -- milliseconds, extra time added for applying less important (de)buffs 
--[[
The spell shader can apply custom effects to the Spell object. The spell object 
contains the texture and font objects, so it's easy to change them based on your
own preferences. 

There is a generic SpellShader effect which is applied to all spells which do not have 
a custom spell shader. To create a custom spell shader for a spell, simply create a 
function in the format SpellShader:_<SPELL_ID>. For example, if I wanted to apply different
heuristics to Bane of Doom, I would create this function

-- 603 is the id of Bane of Doom
function SpellShader:_603()
	self.spell.texture:SetVertexColor(0,1,0); --apply a green mask over the texture.
	self.spell.disableStacks = true
end

For special occasions, you can also create _<SPELL_ID>_init() function which gets executed on
object creation for your spell.

]]--
function SpellShader:new(spell)
	local object = { spell = spell, doShade = nil }
	local funcname = "_";
	local funcname_init = nil;
	if spell.id then
		funcname = funcname .. spell.id;
		funcname_init = funcname .. "_init";
	end

	setmetatable (object, { __index = SpellShader } )
	
	if self[funcname] == nil then
		object.doShade = self["generic"];
	else
		object.doShade = self[funcname];
	end
	
	if self[funcname_init] ~= nil then
		self[funcname_init](object);
	end
	return object
end

function SpellShader:generic()
	local cast_threshold = 0;
	if self.spell.props.spell_type == DOT then
		local down, up, lagHome, lagWorld = GetNetStats();
		-- patch 6.0.2 allows refreshing dots at the 30% mark of the duration
		cast_threshold = self.spell.totalDuration * 0.3 + self.spell.props.castTime + lagWorld;	
		if self.spell.timeleft <= self.spell.props.castTime + lagWorld then
			--dot will drop off			
			self.spell.texture:SetVertexColor(1,0,0);
			self.spell.texture:SetDesaturated(false);
			self.spell.texture:SetAlpha(1);
			self:glow(true);
		elseif self.spell.timeleft <= cast_threshold then
			--optimum time to cast gogogo
			self.spell.texture:SetDesaturated(false);
			self.spell.texture:SetAlpha(1);
			self.spell.texture:SetVertexColor(1,1,1);
			self:glow(true);
		else
			--it will be a while before it drops off
			self.spell.texture:SetVertexColor(1,1,1);
			self.spell.texture:SetAlpha(SPELL_OPACITY);
			self.spell.texture:SetDesaturated(true);
			self:glow(false);
		end
	elseif self.spell.props.spell_type == DEBUFF or self.spell.props.spell_type == BUFF then
		if self.spell.props.castTime then
			local down, up, lagHome, lagWorld = GetNetStats();
			cast_threshold = self.spell.props.castTime + lagWorld + BUFFER_TIME;	
			if self.spell.timeleft <= cast_threshold then			
				--optimum time to cast gogogo
				self.spell.texture:SetDesaturated(false);
				self.spell.texture:SetAlpha(1);
				self.spell.texture:SetVertexColor(1,1,1);
				self:glow(true);
			else
				self:glow(false);
				--it will be a while before it drops off
				self.spell.texture:SetVertexColor(1,1,1);
				self.spell.texture:SetAlpha(SPELL_OPACITY);
				self.spell.texture:SetDesaturated(true);
			end
		end
	end
end

function SpellShader:glow(enabled)
	if self.spell.disable_glow == true or self.spell.action_button_id == nil then
		return;
	end
	local button = _G[self.spell.action_button_id];
	if button == nil then
		-- print('button == nil! ' .. self.spell.action_button_id);
		return;
	end
	
	local cd = GetSpellCooldown(self.spell.action_button_spell_id);
	if enabled and IsUsableAction(button.action) and cd == 0 then
		ActionButton_ShowOverlayGlow(button);
	else
		ActionButton_HideOverlayGlow(button);
	end
end